@@ -47,6 +47,7 @@
 #include <sys/types.h>
 
 #include <assert.h>
+#include <ctype.h>
 #include <dirent.h>
 #include <errno.h>
 #include <limits.h>
@@ -129,12 +130,11 @@ create_mutex (CK_VOID_PTR_PTR mut)
 {
 	mutex_t *pmutex;
 
-	if (mut == NULL)
-		return CKR_ARGUMENTS_BAD;
+	return_val_if_fail (mut != NULL, CKR_ARGUMENTS_BAD);
 
 	pmutex = malloc (sizeof (mutex_t));
-	if (!pmutex)
-		return CKR_HOST_MEMORY;
+	return_val_if_fail (pmutex != NULL, CKR_HOST_MEMORY);
+
 	_p11_mutex_init (pmutex);
 	*mut = pmutex;
 	return CKR_OK;
@@ -145,8 +145,7 @@ destroy_mutex (CK_VOID_PTR mut)
 {
 	mutex_t *pmutex = mut;
 
-	if (mut == NULL)
-		return CKR_MUTEX_BAD;
+	return_val_if_fail (mut != NULL, CKR_MUTEX_BAD);
 
 	_p11_mutex_uninit (pmutex);
 	free (pmutex);
@@ -158,8 +157,7 @@ lock_mutex (CK_VOID_PTR mut)
 {
 	mutex_t *pmutex = mut;
 
-	if (mut == NULL)
-		return CKR_MUTEX_BAD;
+	return_val_if_fail (mut != NULL, CKR_MUTEX_BAD);
 
 	_p11_mutex_lock (pmutex);
 	return CKR_OK;
@@ -170,8 +168,7 @@ unlock_mutex (CK_VOID_PTR mut)
 {
 	mutex_t *pmutex = mut;
 
-	if (mut == NULL)
-		return CKR_MUTEX_BAD;
+	return_val_if_fail (mut != NULL, CKR_MUTEX_BAD);
 
 	_p11_mutex_unlock (pmutex);
 	return CKR_OK;
@@ -182,7 +179,7 @@ free_module_unlocked (void *data)
 {
 	Module *mod = data;
 
-	assert (mod);
+	assert (mod != NULL);
 
 	/* Module must be finalized */
 	assert (mod->initialize_called == 0);
@@ -200,14 +197,13 @@ free_module_unlocked (void *data)
 	free (mod);
 }
 
-static Module*
+static Module *
 alloc_module_unlocked (void)
 {
 	Module *mod;
 
 	mod = calloc (1, sizeof (Module));
-	if (!mod)
-		return NULL;
+	return_val_if_fail (mod != NULL, NULL);
 
 	mod->init_args.CreateMutex = create_mutex;
 	mod->init_args.DestroyMutex = destroy_mutex;
@@ -237,16 +233,15 @@ build_path (const char *dir, const char *filename)
 	assert (filename);
 
 	len = snprintf (NULL, 0, "%s/%s", dir, filename) + 1;
-	if (len <= 0)
-		return NULL;
+	return_val_if_fail (len > 0, NULL);
 
 #ifdef PATH_MAX
 	if (len > PATH_MAX)
 		return NULL;
 #endif
 
-	if (!(path = malloc (len)))
-		return NULL;
+	path = malloc (len);
+	return_val_if_fail (path != NULL, NULL);
 
 	sprintf (path, "%s/%s", dir, filename);
 
@@ -294,8 +289,7 @@ load_module_from_file_unlocked (const char *path, Module **result)
 	CK_RV rv;
 
 	mod = alloc_module_unlocked ();
-	if (!mod)
-		return CKR_HOST_MEMORY;
+	return_val_if_fail (mod != NULL, CKR_HOST_MEMORY);
 
 	rv = dlopen_and_get_function_list (mod, path);
 	if (rv != CKR_OK) {
@@ -312,8 +306,7 @@ load_module_from_file_unlocked (const char *path, Module **result)
 		mod = prev;
 
 	} else if (!_p11_hash_set (gl.modules, mod->funcs, mod)) {
-		free_module_unlocked (mod);
-		return CKR_HOST_MEMORY;
+		return_val_if_reached (CKR_HOST_MEMORY);
 	}
 
 	if (result)
@@ -336,6 +329,61 @@ expand_module_path (const char *filename)
 	return path;
 }
 
+static int
+is_list_delimiter (char ch)
+{
+	return ch == ',' ||  isspace (ch);
+}
+
+static int
+is_string_in_list (const char *list,
+                   const char *string)
+{
+	const char *where;
+
+	where = strstr (list, string);
+	if (where == NULL)
+		return 0;
+
+	/* Has to be at beginning/end of string, and delimiter before/after */
+	if (where != list && !is_list_delimiter (*(where - 1)))
+		return 0;
+
+	where += strlen (string);
+	return (*where == '\0' || is_list_delimiter (*where));
+}
+
+static int
+is_module_enabled_unlocked (const char *name,
+                            hashmap *config)
+{
+	const char *progname;
+	const char *enable_in;
+	const char *disable_in;
+	int enable;
+
+	enable_in = _p11_hash_get (config, "enable-in");
+	disable_in = _p11_hash_get (config, "disable-in");
+
+	/* Defaults to enabled if neither of these are set */
+	if (!enable_in && !disable_in)
+		return 1;
+
+	progname = _p11_get_progname_unlocked ();
+	if (enable_in && disable_in)
+		_p11_message ("module '%s' has both enable-in and disable-in options", name);
+	if (enable_in)
+		enable = (progname != NULL && is_string_in_list (enable_in, progname));
+	else if (disable_in)
+		enable = (progname == NULL || !is_string_in_list (disable_in, progname));
+
+	_p11_debug ("%s module '%s' running in '%s'",
+	            enable ? "enabled" : "disabled",
+	            name,
+	            progname ? progname : "(null)");
+	return enable;
+}
+
 static CK_RV
 take_config_and_load_module_unlocked (char **name, hashmap **config)
 {
@@ -349,6 +397,9 @@ take_config_and_load_module_unlocked (char **name, hashmap **config)
 	assert (config);
 	assert (*config);
 
+	if (!is_module_enabled_unlocked (*name, *config))
+		return CKR_OK;
+
 	module_filename = _p11_hash_get (*config, "module");
 	if (module_filename == NULL) {
 		_p11_debug ("no module path for module, skipping: %s", *name);
@@ -356,18 +407,14 @@ take_config_and_load_module_unlocked (char **name, hashmap **config)
 	}
 
 	path = expand_module_path (module_filename);
-	if (!path)
-		return CKR_HOST_MEMORY;
+	return_val_if_fail (path != NULL, CKR_HOST_MEMORY);
 
 	/* The hash map will take ownership of the variable */
-	if (!_p11_hash_set (*config, "module", path)) {
-		free (path);
-		return CKR_HOST_MEMORY;
-	}
+	if (!_p11_hash_set (*config, "module", path))
+		return_val_if_reached (CKR_HOST_MEMORY);
 
 	mod = alloc_module_unlocked ();
-	if (!mod)
-		return CKR_HOST_MEMORY;
+	return_val_if_fail (mod != NULL, CKR_HOST_MEMORY);
 
 	/* Take ownership of thes evariables */
 	mod->config = *config;
@@ -381,6 +428,13 @@ take_config_and_load_module_unlocked (char **name, hashmap **config)
 		return rv;
 	}
 
+	/*
+	 * We support setting of CK_C_INITIALIZE_ARGS.pReserved from
+	 * 'x-init-reserved' setting in the config. This only works with specific
+	 * PKCS#11 modules, and is non-standard use of that field.
+	 */
+	mod->init_args.pReserved = _p11_hash_get (mod->config, "x-init-reserved");
+
 	prev = _p11_hash_get (gl.modules, mod->funcs);
 
 	/* If same module was loaded previously, just take over config */
@@ -398,18 +452,9 @@ take_config_and_load_module_unlocked (char **name, hashmap **config)
 
 	/* Add this new module to our hash table */
 	} else {
-		if (!_p11_hash_set (gl.modules, mod->funcs, mod)) {
-			free_module_unlocked (mod);
-			return CKR_HOST_MEMORY;
-		}
-
+		if (!_p11_hash_set (gl.modules, mod->funcs, mod))
+			return_val_if_reached (CKR_HOST_MEMORY);
 	}
-	/*
-	 * We support setting of CK_C_INITIALIZE_ARGS.pReserved from
-	 * 'x-init-reserved' setting in the config. This only works with specific
-	 * PKCS#11 modules, and is non-standard use of that field.
-	 */
-	mod->init_args.pReserved = _p11_hash_get (mod->config, "x-init-reserved");
 
 	return CKR_OK;
 }
@@ -432,14 +477,14 @@ load_registered_modules_unlocked (void)
 	/* Load the global configuration files */
 	config = _p11_conf_load_globals (P11_SYSTEM_CONFIG_FILE, P11_USER_CONFIG_FILE, &mode);
 	if (config == NULL)
-		return (errno == ENOMEM) ? CKR_HOST_MEMORY : CKR_GENERAL_ERROR;
+		return CKR_GENERAL_ERROR;
 
 	assert (mode != CONF_USER_INVALID);
 
 	configs = _p11_conf_load_modules (mode, P11_SYSTEM_CONFIG_MODULES,
 	                                  P11_USER_CONFIG_MODULES);
 	if (configs == NULL) {
-		rv = (errno == ENOMEM) ? CKR_HOST_MEMORY : CKR_GENERAL_ERROR;
+		rv = CKR_GENERAL_ERROR;
 		_p11_hash_free (config);
 		return rv;
 	}
@@ -454,7 +499,7 @@ load_registered_modules_unlocked (void)
 	_p11_hash_iterate (configs, &iter);
 	while (_p11_hash_next (&iter, &key, NULL)) {
 		if (!_p11_hash_steal (configs, key, (void**)&name, (void**)&config))
-			assert (0 && "not reached");
+			assert_not_reached ();
 
 		/* Is this a critical module, should abort loading of others? */
 		critical = _p11_conf_parse_boolean (_p11_hash_get (config, "critical"), 0);
@@ -465,14 +510,17 @@ load_registered_modules_unlocked (void)
 		 * These variables will be cleared if ownership is transeferred
 		 * by the above function call.
 		 */
-		free (name);
 		_p11_hash_free (config);
 
 		if (critical && rv != CKR_OK) {
-			_p11_message ("aborting initializationg because module '%s' was marked as critical");
+			_p11_message ("aborting initialization because module '%s' was marked as critical",
+			              name);
 			_p11_hash_free (configs);
+			free (name);
 			return rv;
 		}
+
+		free (name);
 	}
 
 	_p11_hash_free (configs);
@@ -541,21 +589,14 @@ reinitialize_after_fork (void)
 	hashiter iter;
 	Module *mod;
 
-	/* WARNING: This function must be reentrant */
 	_p11_debug ("forked");
 
 	_p11_lock ();
 
 		if (gl.modules) {
 			_p11_hash_iterate (gl.modules, &iter);
-			while (_p11_hash_next (&iter, NULL, (void **)&mod)) {
-				if (mod->initialize_called) {
-					mod->initialize_called = 0;
-
-					/* WARNING: Reentrancy can occur here */
-					initialize_module_unlocked_reentrant (mod);
-				}
-			}
+			while (_p11_hash_next (&iter, NULL, (void **)&mod))
+				mod->initialize_called = 0;
 		}
 
 	_p11_unlock ();
@@ -570,11 +611,11 @@ init_globals_unlocked (void)
 {
 	static int once = 0;
 
-	if (!gl.modules)
+	if (!gl.modules) {
 		gl.modules = _p11_hash_create (_p11_hash_direct_hash, _p11_hash_direct_equal,
 		                          NULL, free_module_unlocked);
-	if (!gl.modules)
-		return CKR_HOST_MEMORY;
+		return_val_if_fail (gl.modules != NULL, CKR_HOST_MEMORY);
+	}
 
 	if (once)
 		return CKR_OK;
@@ -669,6 +710,7 @@ _p11_kit_initialize_registered_unlocked_reentrant (void)
 {
 	Module *mod;
 	hashiter iter;
+	int critical;
 	CK_RV rv;
 
 	rv = init_globals_unlocked ();
@@ -681,15 +723,22 @@ _p11_kit_initialize_registered_unlocked_reentrant (void)
 		while (_p11_hash_next (&iter, NULL, (void **)&mod)) {
 
 			/* Skip all modules that aren't registered */
-			if (!mod->name)
+			if (mod->name == NULL || !is_module_enabled_unlocked (mod->name, mod->config))
 				continue;
 
 			rv = initialize_module_unlocked_reentrant (mod);
 
+			/*
+			 * Module failed to initialize. If this is a critical module,
+			 * then this, should abort loading of others.
+			 */
 			if (rv != CKR_OK) {
-				_p11_debug ("failed to initialize module: %s: %s",
-				       mod->name, p11_kit_strerror (rv));
-				break;
+				critical = _p11_conf_parse_boolean (_p11_hash_get (mod->config, "critical"), 0);
+				if (critical) {
+					_p11_debug ("failed to initialize module: %s: %s",
+					            mod->name, p11_kit_strerror (rv));
+					break;
+				}
 			}
 		}
 	}
@@ -834,13 +883,28 @@ _p11_kit_registered_modules_unlocked (void)
 	hashiter iter;
 	int i = 0;
 
-	if (gl.modules)
+	if (gl.modules) {
 		result = calloc (_p11_hash_size (gl.modules) + 1, sizeof (CK_FUNCTION_LIST_PTR));
-	if (result) {
+		return_val_if_fail (result != NULL, NULL);
+
 		_p11_hash_iterate (gl.modules, &iter);
 		while (_p11_hash_next (&iter, NULL, (void **)&mod)) {
-			if (mod->ref_count && mod->name)
+
+			/*
+			 * We don't include unreferenced modules. We don't include
+			 * modules that have been initialized but aren't in the
+			 * registry. These have a NULL name.
+			 *
+			 * In addition we check again that the module isn't disabled
+			 * using enable-in or disable-in. This is because a caller
+			 * can change the progname we recognize the process as after
+			 * having initialized. This is a corner case, but want to make
+			 * sure to cover it.
+			 */
+			if (mod->ref_count && mod->name &&
+			    is_module_enabled_unlocked (mod->name, mod->config)) {
 				result[i++] = mod->funcs;
+			}
 		}
 	}
 
@@ -896,6 +960,8 @@ p11_kit_registered_module_to_name (CK_FUNCTION_LIST_PTR module)
 	Module *mod;
 	char *name = NULL;
 
+	return_val_if_fail (module != NULL, NULL);
+
 	_p11_library_init_once ();
 
 	_p11_lock ();
@@ -927,13 +993,15 @@ p11_kit_registered_name_to_module (const char *name)
 	CK_FUNCTION_LIST_PTR module = NULL;
 	Module *mod;
 
+	return_val_if_fail (name != NULL, NULL);
+
 	_p11_lock ();
 
 		_p11_kit_clear_message ();
 
 		if (gl.modules) {
 			mod = find_module_for_name_unlocked (name);
-			if (mod)
+			if (mod != NULL && is_module_enabled_unlocked (name, mod->config))
 				module = mod->funcs;
 		}
 
@@ -962,6 +1030,8 @@ p11_kit_registered_option (CK_FUNCTION_LIST_PTR module, const char *field)
 	char *option = NULL;
 	hashmap *config = NULL;
 
+	return_val_if_fail (field != NULL, NULL);
+
 	_p11_library_init_once ();
 
 	_p11_lock ();
@@ -1025,6 +1095,8 @@ p11_kit_initialize_module (CK_FUNCTION_LIST_PTR module)
 	Module *mod;
 	CK_RV rv = CKR_OK;
 
+	return_val_if_fail (module != NULL, CKR_ARGUMENTS_BAD);
+
 	_p11_library_init_once ();
 
 	/* WARNING: This function must be reentrant for the same arguments */
@@ -1110,6 +1182,8 @@ p11_kit_finalize_module (CK_FUNCTION_LIST_PTR module)
 	Module *mod;
 	CK_RV rv = CKR_OK;
 
+	return_val_if_fail (module != NULL, CKR_ARGUMENTS_BAD);
+
 	_p11_library_init_once ();
 
 	/* WARNING: This function must be reentrant for the same arguments */
@@ -1176,6 +1250,9 @@ p11_kit_load_initialize_module (const char *module_path,
 	Module *mod;
 	CK_RV rv = CKR_OK;
 
+	return_val_if_fail (module_path != NULL, CKR_ARGUMENTS_BAD);
+	return_val_if_fail (module != NULL, CKR_ARGUMENTS_BAD);
+
 	_p11_library_init_once ();
 
 	/* WARNING: This function must be reentrant for the same arguments */
