@@ -38,6 +38,7 @@
 #define DEBUG_FLAG DEBUG_PROXY
 #include "debug.h"
 #include "hashmap.h"
+#define CRYPTOKI_EXPORTS
 #include "pkcs11.h"
 #include "p11-kit.h"
 #include "private.h"
@@ -46,7 +47,6 @@
 #include <sys/types.h>
 #include <assert.h>
 #include <errno.h>
-#include <pthread.h>
 #include <stdarg.h>
 #include <stddef.h>
 #include <stdlib.h>
@@ -147,7 +147,7 @@ map_session_to_real (CK_SESSION_HANDLE_PTR handle, Mapping *mapping, Session *se
 			rv = CKR_CRYPTOKI_NOT_INITIALIZED;
 		} else {
 			assert (gl.sessions);
-			sess = hash_get (gl.sessions, handle);
+			sess = _p11_hash_get (gl.sessions, handle);
 			if (sess != NULL) {
 				*handle = sess->real_session;
 				rv = map_slot_unlocked (sess->wrap_slot, mapping);
@@ -177,7 +177,7 @@ finalize_mappings_unlocked (void)
 	gl.n_mappings = 0;
 
 	/* no more sessions */
-	hash_free (gl.sessions);
+	_p11_hash_free (gl.sessions);
 	gl.sessions = NULL;
 }
 
@@ -204,7 +204,7 @@ proxy_C_Finalize (CK_VOID_PTR reserved)
 {
 	CK_RV rv;
 
-	debug ("in");
+	_p11_debug ("in");
 
 	/* WARNING: This function must be reentrant */
 
@@ -227,7 +227,7 @@ proxy_C_Finalize (CK_VOID_PTR reserved)
 		_p11_unlock ();
 	}
 
-	debug ("out: %lu", rv);
+	_p11_debug ("out: %lu", rv);
 	return rv;
 }
 
@@ -257,10 +257,7 @@ initialize_mappings_unlocked_reentrant (void)
 			rv = (funcs->C_GetSlotList) (FALSE, NULL, &count);
 			if (rv == CKR_OK && count) {
 				slots = calloc (sizeof (CK_SLOT_ID), count);
-				if (!slots)
-					rv = CKR_HOST_MEMORY;
-				else
-					rv = (funcs->C_GetSlotList) (FALSE, slots, &count);
+				rv = (funcs->C_GetSlotList) (FALSE, slots, &count);
 			}
 
 		_p11_lock ();
@@ -270,12 +267,10 @@ initialize_mappings_unlocked_reentrant (void)
 			break;
 		}
 
-		mappings = xrealloc (mappings, sizeof (Mapping) * (n_mappings + count));
-		if (!mappings) {
-			free (slots);
-			rv = CKR_HOST_MEMORY;
-			break;
-		}
+		return_val_if_fail (count == 0 || slots != NULL, CKR_GENERAL_ERROR);
+
+		mappings = realloc (mappings, sizeof (Mapping) * (n_mappings + count));
+		return_val_if_fail (mappings != NULL, CKR_HOST_MEMORY);
 
 		/* And now add a mapping for each of those slots */
 		for (i = 0; i < count; ++i) {
@@ -297,7 +292,7 @@ initialize_mappings_unlocked_reentrant (void)
 	assert (!gl.sessions);
 	gl.mappings = mappings;
 	gl.n_mappings = n_mappings;
-	gl.sessions = hash_create (hash_ulongptr_hash, hash_ulongptr_equal, NULL, free);
+	gl.sessions = _p11_hash_create (_p11_hash_ulongptr_hash, _p11_hash_ulongptr_equal, NULL, free);
 	++gl.mappings_refs;
 
 	/* Any cleanup necessary for failure will happen at caller */
@@ -309,9 +304,11 @@ proxy_C_Initialize (CK_VOID_PTR init_args)
 {
 	CK_RV rv;
 
+	_p11_library_init_once ();
+
 	/* WARNING: This function must be reentrant */
 
-	debug ("in");
+	_p11_debug ("in");
 
 	_p11_lock ();
 
@@ -324,12 +321,12 @@ proxy_C_Initialize (CK_VOID_PTR init_args)
 
 	_p11_unlock ();
 
-	debug ("here");
+	_p11_debug ("here");
 
 	if (rv != CKR_OK)
 		proxy_C_Finalize (NULL);
 
-	debug ("out: %lu", rv);
+	_p11_debug ("out: %lu", rv);
 	return rv;
 }
 
@@ -338,8 +335,9 @@ proxy_C_GetInfo (CK_INFO_PTR info)
 {
 	CK_RV rv = CKR_OK;
 
-	if (info == NULL)
-		return CKR_ARGUMENTS_BAD;
+	_p11_library_init_once ();
+
+	return_val_if_fail (info != NULL, CKR_ARGUMENTS_BAD);
 
 	_p11_lock ();
 
@@ -366,8 +364,7 @@ proxy_C_GetFunctionList (CK_FUNCTION_LIST_PTR_PTR list)
 {
 	/* Can be called before C_Initialize */
 
-	if (!list)
-		return CKR_ARGUMENTS_BAD;
+	return_val_if_fail (list != NULL, CKR_ARGUMENTS_BAD);
 	*list = &proxy_function_list;
 	return CKR_OK;
 }
@@ -382,8 +379,7 @@ proxy_C_GetSlotList (CK_BBOOL token_present, CK_SLOT_ID_PTR slot_list,
 	CK_RV rv = CKR_OK;
 	int i;
 
-	if (!count)
-		return CKR_ARGUMENTS_BAD;
+	return_val_if_fail (count != NULL, CKR_ARGUMENTS_BAD);
 
 	_p11_lock ();
 
@@ -499,8 +495,7 @@ proxy_C_OpenSession (CK_SLOT_ID id, CK_FLAGS flags, CK_VOID_PTR user_data,
 	Mapping map;
 	CK_RV rv;
 
-	if (handle == NULL)
-		return CKR_ARGUMENTS_BAD;
+	return_val_if_fail (handle != NULL, CKR_ARGUMENTS_BAD);
 
 	rv = map_slot_to_real (&id, &map);
 	if (rv != CKR_OK)
@@ -525,7 +520,7 @@ proxy_C_OpenSession (CK_SLOT_ID id, CK_FLAGS flags, CK_VOID_PTR user_data,
 				sess->wrap_slot = map.wrap_slot;
 				sess->real_session = *handle;
 				sess->wrap_session = ++gl.last_handle; /* TODO: Handle wrapping, and then collisions */
-				hash_set (gl.sessions, &sess->wrap_session, sess);
+				_p11_hash_set (gl.sessions, &sess->wrap_session, sess);
 				*handle = sess->wrap_session;
 			}
 
@@ -552,7 +547,7 @@ proxy_C_CloseSession (CK_SESSION_HANDLE handle)
 		_p11_lock ();
 
 			if (gl.sessions)
-				hash_remove (gl.sessions, &key);
+				_p11_hash_remove (gl.sessions, &key);
 
 		_p11_unlock ();
 	}
@@ -574,13 +569,13 @@ proxy_C_CloseAllSessions (CK_SLOT_ID id)
 		if (!gl.sessions) {
 			rv = CKR_CRYPTOKI_NOT_INITIALIZED;
 		} else {
-			to_close = calloc (sizeof (CK_SESSION_HANDLE), hash_size (gl.sessions));
+			to_close = calloc (sizeof (CK_SESSION_HANDLE), _p11_hash_size (gl.sessions));
 			if (!to_close) {
 				rv = CKR_HOST_MEMORY;
 			} else {
-				hash_iterate (gl.sessions, &iter);
+				_p11_hash_iterate (gl.sessions, &iter);
 				count = 0;
-				while (hash_next (&iter, NULL, (void**)&sess)) {
+				while (_p11_hash_next (&iter, NULL, (void**)&sess)) {
 					if (sess->wrap_slot == id && to_close)
 						to_close[count++] = sess->wrap_session;
 				}
@@ -1389,8 +1384,13 @@ static CK_FUNCTION_LIST proxy_function_list = {
 	proxy_C_WaitForSlotEvent
 };
 
+#ifdef OS_WIN32
+__declspec(dllexport)
+#endif
+
 CK_RV
 C_GetFunctionList (CK_FUNCTION_LIST_PTR_PTR list)
 {
+	_p11_library_init_once ();
 	return proxy_C_GetFunctionList (list);
 }
