@@ -1,5 +1,6 @@
 /*
  * Copyright (c) 2011 Collabora Ltd
+ * Copyright (c) 2012 Stef Walter
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -31,11 +32,12 @@
  *
  *
  * CONTRIBUTORS
- *  Stef Walter <stef@memberwebs.com>
+ *  Stef Walter <stef@thewalter.net>
  */
 
 #include "config.h"
 
+#include "compat.h"
 #define DEBUG_FLAG DEBUG_LIB
 #include "debug.h"
 #include "p11-kit.h"
@@ -72,15 +74,6 @@ pthread_once_t _p11_once;
 
 static int print_messages = 1;
 
-void*
-_p11_realloc (void *memory, size_t length)
-{
-	void *allocated = realloc (memory, length);
-	if (!allocated)
-		free (memory);
-	return allocated;
-}
-
 /**
  * p11_kit_space_strlen:
  * @string: Pointer to string block
@@ -188,6 +181,7 @@ _p11_message (const char* msg, ...)
 	if (print_messages)
 		fprintf (stderr, "p11-kit: %s\n", buffer);
 
+	_p11_debug_message (DEBUG_LIB, "message: %s", buffer);
 	store_message_buffer (buffer, length);
 }
 
@@ -246,6 +240,14 @@ _p11_kit_default_message (CK_RV rv)
 	}
 }
 
+static void
+uninit_common (void)
+{
+	_p11_debug ("uninitializing library");
+
+	_p11_set_progname_unlocked (NULL);
+}
+
 #ifdef OS_UNIX
 
 static pthread_key_t thread_local = 0;
@@ -266,21 +268,47 @@ _p11_library_get_thread_local (void)
 	return local;
 }
 
+#ifdef __GNUC__
+__attribute__((constructor))
+#endif
 void
 _p11_library_init (void)
 {
 	_p11_debug_init ();
+	_p11_debug ("initializing library");
 	_p11_mutex_init (&_p11_mutex);
 	pthread_key_create (&thread_local, free);
 }
 
+#ifdef __GNUC__
+__attribute__((destructor))
+#endif
 void
 _p11_library_uninit (void)
 {
+	uninit_common ();
+
+	/* Some cleanup to pacify valgrind */
+	free (pthread_getspecific (thread_local));
+	pthread_setspecific (thread_local, NULL);
+
 	pthread_key_delete (thread_local);
 	_p11_mutex_uninit (&_p11_mutex);
 }
 
+void
+_p11_mutex_init (mutex_t *mutex)
+{
+	pthread_mutexattr_t attr;
+	int ret;
+
+	pthread_mutexattr_init (&attr);
+	pthread_mutexattr_settype (&attr, PTHREAD_MUTEX_RECURSIVE);
+	ret = pthread_mutex_init (mutex, &attr);
+	assert (ret == 0);
+	pthread_mutexattr_destroy (&attr);
+}
+
 #endif /* OS_UNIX */
 
 #ifdef OS_WIN32
@@ -331,7 +359,7 @@ _p11_library_uninit (void)
 {
 	LPVOID data;
 
-	_p11_debug ("uninitializing library");
+	uninit_common ();
 
 	if (thread_local != TLS_OUT_OF_INDEXES) {
 		data = TlsGetValue (thread_local);
@@ -341,6 +369,59 @@ _p11_library_uninit (void)
 	_p11_mutex_uninit (&_p11_mutex);
 }
 
+const char *
+_p11_module_error (void)
+{
+	DWORD code = GetLastError();
+	p11_local *local;
+	LPVOID msg_buf;
+
+	local = _p11_library_get_thread_local ();
+
+	FormatMessageA (FORMAT_MESSAGE_ALLOCATE_BUFFER |
+	                FORMAT_MESSAGE_FROM_SYSTEM |
+	                FORMAT_MESSAGE_IGNORE_INSERTS,
+	                NULL, code,
+	                MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT),
+	                (LPSTR)&msg_buf, 0, NULL);
+
+	if (local->last_error)
+		LocalFree (local->last_error);
+	local->last_error = msg_buf;
+
+	return msg_buf;
+}
+
+int
+_p11_thread_create (thread_t *thread,
+               thread_routine routine,
+               void *arg)
+{
+	assert (thread);
+
+	*thread = CreateThread (NULL, 0,
+	                        (LPTHREAD_START_ROUTINE)routine,
+	                        arg, 0, NULL);
+
+	if (*thread == NULL)
+		return GetLastError ();
+
+	return 0;
+}
+
+int
+_p11_thread_join (thread_t thread)
+{
+	DWORD res;
+
+	res = WaitForSingleObject (thread, INFINITE);
+	if (res == WAIT_FAILED)
+		return GetLastError ();
+
+	CloseHandle (thread);
+	return 0;
+}
+
 BOOL WINAPI
 DllMain (HINSTANCE instance,
          DWORD reason,
@@ -377,3 +458,42 @@ DllMain (HINSTANCE instance,
 }
 
 #endif /* OS_WIN32 */
+
+/* This is the progname that we think of this process as. */
+char *_p11_my_progname = NULL;
+
+/**
+ * p11_kit_set_progname:
+ * @progname: the program base name
+ *
+ * Set the program base name that is used by the <literal>enable-in</literal>
+ * and <literal>disable-in</literal> module configuration options.
+ *
+ * Normally this is automatically calculated from the program's argument list.
+ * You would usually call this before initializing p11-kit modules.
+ */
+void
+p11_kit_set_progname (const char *progname)
+{
+	_p11_library_init_once ();
+
+	_p11_lock ();
+	_p11_set_progname_unlocked (progname);
+	_p11_unlock ();
+}
+
+void
+_p11_set_progname_unlocked (const char *progname)
+{
+	/* We can be called with NULL, to cleanup memory usage */
+	free (_p11_my_progname);
+	_p11_my_progname = progname ? strdup (progname) : NULL;
+}
+
+const char *
+_p11_get_progname_unlocked (void)
+{
+	if (_p11_my_progname == NULL)
+		_p11_set_progname_unlocked (getprogname ());
+	return _p11_my_progname;
+}
